package com.sleepycat.je.evictor;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.utilint.DaemonThread;

public class EvictorDaemon extends DaemonThread {
	public static final String SOURCE_DAEMON = "daemon";

	private Evictor evictor;
	private EnvironmentImpl envImpl;

	public EvictorDaemon(Evictor evictor, EnvironmentImpl envImpl, String name) {
		super(0, name, envImpl);
		this.evictor=evictor;
		this.envImpl=envImpl;
	}
	
	
	/**
	 * Evictor doesn't have a work queue so just throw an exception if it's ever
	 * called.
	 */
	public void addToQueue(Object o) throws DatabaseException {

		throw new DatabaseException(
				"Evictor.addToQueue should never be called.");
	}
	/**
	 * Return the number of retries when a deadlock exception occurs.
	 */
	protected int nDeadlockRetries() throws DatabaseException {

		return envImpl.getConfigManager().getInt(
				EnvironmentParams.EVICTOR_RETRY);
	}
	
	/**
	 * Called whenever the daemon thread wakes up from a sleep.
	 */
	public void onWakeup() throws DatabaseException {

		if (envImpl.isClosed()) {
			return;
		}

		evictor.doEvict(SOURCE_DAEMON);
	}
	
	/**
	 * Wakeup the evictor only if it's not already active.
	 */
	public void alert() {
		if (!evictor.isActive()) {
			wakeup();
		}
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("<EvictorDaemon name=\"").append(name).append("\"/>");
		return sb.toString();
	}
	

	synchronized public void clearEnv() {
		envImpl = null;
	}
}
