package com.sleepycat.je.evictor;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.BooleanConfigParam;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.dbi.DbConfigManager;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.dbi.MemoryBudget;

public privileged aspect WeaveEvictorDaemon {
	
	
	/* @deprecated As of 2.0, eviction is performed in-line. */
	public static final BooleanConfigParam EnvironmentParams.ENV_RUN_EVICTOR = new BooleanConfigParam(
			"je.env.runEvictor", false, // default
			true, // mutable
			"# If true, starts up the evictor.\n"
					+ "# This parameter is false by default\n"
					+ "# (deprecated, eviction is performed in-line");


	private EvictorDaemon EnvironmentImpl.evictorDaemon;
	
	declare precedence: *, WeaveEvictorDaemon;
	
	after(EnvironmentImpl env) throws InterruptedException :
		execution(void EnvironmentImpl.shutdownDaemons()) && this(env){
		
		/*
		 * The evictor has to get shutdown last because the other daemons might
		 * create changes to the memory usage which result in a notify to
		 * eviction.
		 */
		env.shutdownEvictorDaemon();
  }
  
	public void EnvironmentImpl.shutdownEvictorDaemon() throws InterruptedException {

		if (evictorDaemon != null) {
			evictorDaemon.shutdown();

			/*
			 * If daemon thread doesn't shutdown for any reason, at least clear
			 * the reference to the environment so it can be GC'd.
			 */
			evictorDaemon.clearEnv();
			evictorDaemon = null;
		}
		return;
	}
	
	void EnvironmentImpl.alertEvictorDaemon() {
		if (evictorDaemon != null) {
			evictorDaemon.alert();
		}
	}
	
	before(EnvironmentImpl env):
		execution(void EnvironmentImpl.requestShutdownDaemons()) && target(env){
		
		if (env.evictorDaemon != null) {
			env.evictorDaemon.requestShutdown();
		}
	}
	
	after(EnvironmentImpl env,DbConfigManager mgr) throws DatabaseException:
		execution(void EnvironmentImpl.runOrPauseDaemons(DbConfigManager)) && this(env) && args(mgr){
		
		/* Evictor */
		env.evictorDaemon.runOrPause(mgr.getBoolean(EnvironmentParams.ENV_RUN_EVICTOR));
	}
	
	before(EnvironmentImpl env):
		execution(void EnvironmentImpl.createDaemons()) && this(env){
		
		/* Evictor */
		env.evictorDaemon = new EvictorDaemon(env.evictor,env, "Evictor");
	}
	
	after(MemoryBudget mb):execution(void MemoryBudget.update*MemoryUsage(..)) && this(mb) {
		if (mb.getCacheMemoryUsage() > mb.cacheBudget) {
			mb.envImpl.alertEvictorDaemon();
		}
	}
}
